version 18 

clear all 

set seed 111
set obs 1000 
set scheme stcolor 

generate e0    = rchi2(5)-5
generate e1    = rchi2(5)-5 
generate x1    = rbeta(3,1)*400
generate x2    = rbeta(3,1)*400
generate y0    = 10 - x1 + x2 + e0
generate y1    = 2*(10 - x1 + x2) + e1
generate treat = runiform()>.4
generate y     = treat*y1 + (1-treat)*y0 
generate te    = y1 - y0

list y treat y1 y0 te in 1/10, noobs sep(0)

sysuse auto, clear 
summarize i.foreign
  
summarize i.rep78
  
summarize i.rep78#i.foreign
summarize rep78#foreign
 

summarize i.rep78##i.foreign
 

summarize c.mpg#i.foreign
summarize c.mpg##i.foreign
summarize c.mpg##c.mpg 

webuse cattaneo2, clear
describe *weight* m* alcohol f* 


twoway scatter bweight mage if mbsmoke==1,		    ///				
               msymbol(oh) mcolor(navy)             ///
    || scatter bweight mage if mbsmoke==0,          ///
	           msymbol(oh) mcolor(maroon)			///                   
			legend(label(1 bweight smk)) 		    ///
			legend(label(2 bweight nonsmk))			///
			legend(position(12) ring(0) rows(1)     ///
			region(lstyle(none)))				    ///
			ytitle("Birthweight in grams" " ")
			
graph export ragraph0.png, replace

twoway scatter bweight mage if mbsmoke==1,		    ///				
               msymbol(oh) mcolor(navy)             ///
    || scatter bweight mage if mbsmoke==0,          ///
	           msymbol(oh) mcolor(maroon)			///
	|| lfit bweight mage if mbsmoke==1,			    ///
            lwidth(thick)                           ///
	|| lfit bweight mage if mbsmoke==0,			    ///
	        lwidth(thick)                           ///
			legend(label(1 bweight smk)) 		    ///
			legend(label(2 bweight nonsmk))			///
			legend(label(3 XB smk))					///
			legend(label(4 XB nonsmk))				///
			legend(position(10) ring(0) rows(1)     ///
			region(lstyle(none)))				    ///
			ytitle("Birthweight in grams" " ")
			
graph export ragraph.png, replace

teffects ra (bweight i.prenatal1 i.mmarried mage i.fbaby) (mbsmoke)
estimates store linear
 			
teffects ra (bweight i.prenatal1 i.mmarried mage i.fbaby) (mbsmoke), atet
 
teffects ra (bweight i.prenatal1 i.mmarried mage i.fbaby) (mbsmoke), pom
 
teffects ra (bweight i.prenatal1 i.mmarried mage i.fbaby, poisson) (mbsmoke)
estimates store exponential
 
teffects ra (lbweight i.prenatal1 i.mmarried mage i.fbaby, probit) (mbsmoke)
estimates store probit
 
teffects ra (lbweight i.prenatal1 i.mmarried mage i.fbaby) (mbsmoke)
estimates store linearp
 
estimates table linear exponential, se keep(ATE:)

estimates table probit linearp, se keep(ATE:)

teffects, aequations
 
teffects ra (lbweight i.prenatal1 i.mmarried mage i.fbaby) (mbsmoke),	///
	aequations
 
quietly regress bweight i.prenatal1 i.mmarried mage i.fbaby if mbsmoke==1

predict double te1 

quietly regress bweight i.prenatal1 i.mmarried mage i.fbaby if mbsmoke==0

predict double te0 

generate double teffect = te1 - te0 

summarize teffect 
 
quietly regress bweight ///
	i.mbsmoke##(i.prenatal1 i.mmarried c.mage i.fbaby), vce(robust)
estimates store reg 
margins r.mbsmoke, vce(unconditional) contrast(nowald) noestimcheck

di  23.84972*sqrt((4632)/4642)

local X i.mbsmoke##(i.prenatal1 i.mmarried c.mage i.fbaby)
quietly gmm (bweight -{xb: `X' _cons}),    ///
        instruments(`X') onestep quickderivatives
estimates store gmm 

estimates table gmm reg, eq(1)
 
logit mbsmoke i.mmarried i.alcohol mage fedu
predict double prob 
generate double ps = 1/prob
replace  ps        = 1/(1-prob) if mbsmoke==0 
 

twoway scatter bweight mage [pweight=ps] if mbsmoke==1,		    ///				
               msymbol(oh) mcolor(navy)             ///
    || scatter bweight mage [pweight=ps] if mbsmoke==0,          ///
	           msymbol(oh) mcolor(maroon)			///                   
			legend(label(1 bweight smk)) 		    ///
			legend(label(2 bweight nonsmk))			///
			legend(position(12) ring(0) rows(1)     ///
			region(lstyle(none)))				    ///
			ytitle("Birthweight in grams" " ")
graph export ipwgraph.png, replace

teffects ipw (bweight) (mbsmoke i.mmarried c.mage##c.mage fbaby medu)
estimates store orig
 
teffects ipw (bweight) (mbsmoke i.mmarried c.mage##c.mage fbaby medu, probit)
 
teffects ipw (bweight) (mbsmoke i.mmarried c.mage##c.mage fbaby medu), atet
 
tebalance summarize 

tebalance density mage
graph export tebalance.png, replace 

tebalance density medu
graph export tebalance2.png, replace 

generate mhigh = medu>12 
label var mhigh "some college"
qui teffects ipw (bweight) (mbsmoke i.mmarried c.mage##c.mage fbaby i.mhigh)
estimates store new 
tebalance summarize

tebalance density mhigh
graph export tebalance3.png, replace 

estimates restore orig
tebalance overid 

estimates restore new 
tebalance overid 

clear all 
set obs 5000
set seed 111 

generate x1 = rbeta(2,5) 
generate x2 = (rchi2(5)-5)/sqrt(2*5) 
generate a  = int(rbeta(2,3)*3)
generate e  = rnormal()
generate e0 = rnormal(0, 2)
generate e1 = (rchi2(5)-5)/sqrt(2*5)
generate t  = .5*(1-x1 + x2 -a/2) + e > 0
generate y0 = -.5 - x1*x2 + x1*a - x1 + x2 + a + e0
generate y1 = .5 - x1*x2 + x1*a - x1 + x2 + a + e1
generate y  = t*y1 + (1-t)*y0
 

quietly teffects ra (y c.x1#c.x2 i.a) (t)
estimates store ra
quietly teffects ipw (y) (t x1)
estimates store ipw
quietly teffects ipwra (y c.x1#c.x2 i.a) (t x1 x2 i.a)
estimates store ipwra_ipw
quietly teffects ipwra (y c.x1##(c.x2 i.a)) (t x1)
estimates store ipwra_ra
estimates table ra ipw ipwra*, keep(ATE:) t se
 
quietly teffects ra (y c.x1#c.x2 i.a) (t)
estimates store ra
quietly teffects ipw (y) (t x1)
estimates store ipw
quietly teffects aipw (y c.x1#c.x2 i.a) (t x1 x2 i.a)
estimates store aipw_ipw
quietly teffects aipw (y c.x1##(c.x2 i.a)) (t x1)
estimates store aipw_ra
estimates table ra ipw aipw*, keep(ATE:) t se
 
webuse cattaneo2, clear 

teffects aipw (bweight i.prenatal1 i.mmarried mage i.fbaby)     ///
			  (mbsmoke i.mmarried c.mage##c.mage fbaby medu)

qui logit mbsmoke i.mmarried c.mage##c.mage fbaby medu
qui predict double prob 
qui generate double ps = 1/prob
qui replace  ps        = 1/(1-prob) if mbsmoke==0 
qui regress bweight i.prenatal1 i.mmarried mage i.fbaby [pw=ps] if mbsmoke==1
qui predict double pom_1
qui regress bweight i.prenatal1 i.mmarried mage i.fbaby [pw=ps] if mbsmoke==0
qui predict double pom_0 
teffects ipwra (bweight i.prenatal1 i.mmarried mage i.fbaby)     ///
			  (mbsmoke i.mmarried c.mage##c.mage fbaby medu), pom
sum pom_0 pom_1
 
teffects nnmatch (bweight mage) (mbsmoke)

teffects nnmatch (bweight mage) (mbsmoke), generate(nei)  
list bweight mbsmoke nei1-nei5 in 1/10

teffects psmatch (bweight) (mbsmoke i.mmarried c.mage##c.mage i.fbaby medu)
 
// Homogeneous DID 

clear all 
set obs 100

// Graphs for intro

set seed 111
generate employee = _n
generate offices = runiformint(1,100)
bysort offices: generate oeffect = 1 + rnormal(0, 0.03) //rnormal(1, 0.08) if _n==1
bysort offices: replace  oeffect = oeffect[_n-1] if _n>1
expand 10
bysort employee: generate year = _n + 2005
bysort year: generate yeffect  = rnormal(.05, .04) if _n==1
bysort year: replace  yeffect  = yeffect[_n-1] if _n>1
replace  yeffect  = yeffect -1 if year>2010
sort employee year offices
replace oeffect = oeffect - .3 if offices<=30
generate e  = rnormal(0,.1)
generate y0 = 7 + oeffect + yeffect + e
generate y1 = 7 + oeffect + yeffect + e
generate y2 = 6 + oeffect + yeffect + e
generate policy = 0
replace  policy = 1 if offices<=30 & year>2010
generate y  = y1*policy + y0*(1-policy)
generate yn = y2*policy + y0*(1-policy)
preserve
collapse y y0 y1 y2 yn policy, by(year offices)
xtset offices year
bysort office: egen treated = mean(policy)
bysort office: replace treated = treated>0
bysort year: egen ymeant   = mean(y)  if treated==1
bysort year: egen ymeanc   = mean(y)  if treated==0
bysort year: egen ymeantn  = mean(yn)  if treated==1
bysort year: egen ymeancn  = mean(yn)  if treated==0
bysort year: egen y0mean  = mean(y0)
bysort year: egen y1mean  = mean(y1)
bysort year: egen ymean  = mean(y)
label var year "Year"

twoway connected ymeant year, xline(2010) ytitle("Outcome of interest") ///
        lpattern(dash)

graph export colombia1.png, as(png) replace 
	
label var ymeant  "Treatment"
label var ymeanc  "Control"
label var ymeantn "Treatment"
label var ymeancn "Control"

twoway connected ymeant year, lpattern(dash) ||         ///
       connected ymeanc year, xline(2010)               ///
        ytitle("Outcome of interest") lpattern(dash)
		
graph export colombia2.png, as(png) replace

twoway connected ymeantn year, lpattern(dash) ||        ///
       connected ymeancn year, xline(2010)      ///
        ytitle("Outcome of interest") lpattern(dash)

graph export colombia3.png, as(png) replace

webuse hospdd, clear 
describe 


bysort hospital: egen treated = mean(procedure)
quietly replace treated = 1 if treated>0
bysort month: egen ymeant   = mean(satis)  if treated==1
bysort month: egen ymeanc   = mean(satis)  if treated==0


twoway connected ymeant month, lpattern(dash) xline(3)               ///
        ytitle("Patient satisfaction") lpattern(dash)
		
graph export colombia4.png, as(png) replace
drop treated 

didregress (satis) (procedure), group(hospital) time(month) 

estat trendplot
graph export colombia5.png, as(png) replace
estat ptrends
estat granger


estat grangerplot
graph export grangerplot.png, as(png) replace
estat grangerplot, nodraw verbose
estat grangerplot, lagview
graph export grangerplot2.png, as(png) replace

bysort hospital: egen treated = mean(procedure)
replace treated = 1 if treated>0 
generate post = 0
replace post = 1 if month>3


didregress (satis treated post) (procedure),	///
	group(hospital) time(month) nogteffects
	
generate hightrt = procedure==1 & (frequency==3 | frequency==4)
label define trt 0 "Untreated" 1 "Treated"
label values hightrt trt

didregress (satis) (hightrt), group(hospital frequency) time(month)

didregress (satis) (procedure), ///
	group(hospital) time(month) wildbootstrap(rseed(111))
 
didregress (satis) (procedure), ///
	group(hospital) time(month) wildbootstrap(rseed(111))

didregress (satis) (procedure), group(hospital) time(month) vce(hc2)
mat list r(table)

didregress (satis) (procedure), group(hospital) time(month) aggregate(dlang)

local specs (satis) (procedure)
didregress `specs', group(hospital) time(month) aggregate(standard) vce(hc2)


// Heterogeneous DID

webuse akc, clear
describe

tabulate year movie

xtset breed year 
xthdidregress ra (registered best) (movie), group(breed)

estat atetplot
graph export atetplot.png, as(png) replace

estat aggregation, overall
 
estat aggregation, cohort graph sci
graph export cohort.png, as(png) replace

estat aggregation, cohort(2036 2034) 
 

estat aggregation, time graph sci 
graph export time.png, as(png) replace

 
estat aggregation, dynamic graph 
graph export dynamic.png, as(png) replace

xthdidregress twfe (registered best) (movie), group(breed) hettype(cohort)



